# -*- coding: utf-8 -*-
#get_file_SEVIRI.py SEVIRI_AER_OC Aerosol_Optical_Depth  2009-09-01 2009-09-10 -t 1000 1200
#
import os
import glob
import sys
import os.path

import re
from optparse import OptionParser
from calendar import *
from datetime import datetime,timedelta
from time     import time, strptime, strftime, gmtime
from numpy import *
from pyhdf.SD import*
import warnings
warnings.filterwarnings("ignore")


##### APPLICATION #####
__VERSION__="0.0.0"
__APP__="get_file"
__RUN_PROCESS__=False

##### APP RETURN CODES #####
EXIT_FAILURE=1
EXIT_SUCCESS=0

##### DEFAULT PATHES #####
DEFAULT_INPUT_DIR ="/DATA/LIENS"
DEFAULT_OUTPUT_DIR =os.getcwd()

class ProductionSeviri :

    def __init__(   self,prod_id,dataset,date_start,date_end,subsetting,compress,verbose,stats,command_line,fixed_value,output_id,step,time=None,
		    input_dir = DEFAULT_INPUT_DIR, output_dir = DEFAULT_OUTPUT_DIR):
                    
        products={'SEVIRI_AER_OC':('SEVIRI','AER_OC','SEVIRI_AER_OC_[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9].[0-9][0-9][0-9][0-9].hdf')}
	"""
        Constructor
	"""
        self.step=step
        
        self.output_id=output_id
        
        self.fixed_value=fixed_value
        
        self.verbose=verbose
        
        self.command_line=command_line

	self.target_app = os.path.join( os.path.dirname ( __file__ ), "Average_processor.py" )

        self.Time=self.get_struct_time(date_start,date_end,time)
        
	self.date_start=self.get_date_start(date_start)
	
	self.date_end=self.get_date_end(date_end,date_start)

	self.start_time=self.get_start_time(self.Time[0])

	self.end_time=self.get_end_time(self.Time[-1])

	self.start_time,self.end_time,self.date_start,self.date_end=self.get_time(self.start_time,self.end_time,self.date_start,self.date_end)

	self.dataset=dataset

        self.compress=compress

        self.stats=stats
        
        self.subsetting=subsetting
        
	self.time_flag = self.get_time_flag ( self.date_start,self.date_end )
	
	if ( not os.path.exists ( input_dir ) ) :
            raise ValueError ( "Invalid input directory %s"%input_dir )
        self.input_dir  = input_dir
	
	if ( not os.path.exists ( output_dir ) ) :
            raise ValueError ( "Invalid output directory %s"%output_dir )
        self.output_dir    = output_dir
	
	valid_pror_ids = products.keys() 
	
        if prod_id not in valid_pror_ids :
            raise ValueError ( "Unsupported Product %s Must be in [%s]"%( prod_id, valid_pror_ids ) )
        self.prod_id = prod_id
	
        if self.verbose is True :
        
            print "\n"
            print "  ######################  FICHIERS POUR MOYENNE IMAGES  #######################"
            print "  #                                                                           #"
            print "  #                                ICARE-CGTD				 #"
            print "  #				python "+__APP__+"				 #"
            print "  ############################################################################"
            print "\n"
            print "  -------------------------------------- Parametres ---------------------------"
            print "\n"
            print "\n"
            if self.command_line is not True :
                print "Programme cible		 :",self.target_app
            if time is None :
	      print "CALCUL 	: 	Moyenne du "+self.date_start+" a "+self.start_time[0:2]+'H'+self.start_time[2:]+" au "+self.date_end+" a "+self.end_time[0:2]+'H'+self.end_time[2:]
	    else :
	      print "CALCUL 	:	Moyenne du "+self.date_start+" au "+self.date_end+", pour chaque jour les fichiers lus sont de "+self.start_time[0:2]+'H'+self.start_time[2:]+" a "+self.end_time[0:2]+'H'+self.end_time[2:]
            print "Command_line :",self.command_line
            print "dataset		:",self.dataset
            print "Compress :",compress
            print "Stats :",stats
            print "Subsetting :",subsetting
            print "fixed_value : ",self.fixed_value
            print "self.time_flag :	",self.time_flag
            print "self.input_dir :	",self.input_dir
            print "self.output_dir :	",self.output_dir
            print "Produit SEVIRI 		 :",self.prod_id
            
        
	v_prod_id = [ self.prod_id ]
	for prod_id in iter ( v_prod_id ) : 
            # build the list of files timestamps matching the time range, and extract the timestamp from the list of files
            list_files= self.get_infiles ( prod_id,time,products )
           
            # run target application on the given product and the given timestamps
	    if len(list_files)!=0 : 
            
                # creation de la liste pour le pcf
                list_pcf={}   
                list_pcf['Input_files']=list_files
                list_pcf['dataset']=self.dataset
                list_pcf['output_directory']=self.output_dir
                
                if self.subsetting is None : list_pcf['subsetting']=str(None)
                else : list_pcf['subsetting']=str(self.subsetting)
                
                if self.fixed_value is not None : list_pcf['fixed_value']=str(self.fixed_value)
                
                list_pcf['valmin']=str(None)
                list_pcf['valmax']=str(None)
                list_pcf['compress']=str(self.compress)
                list_pcf['verbose']=str(self.verbose)
                list_pcf['stats']=str(self.stats)
                list_pcf['command_line']=str(self.command_line)

                self.run ( prod_id, list_files,time,list_pcf,products)
	    else :
	      sensor, directory, pattern = products [ prod_id ]
	      raise ValueError ( "No file found in the archive, "+ os.path.join ( self.input_dir,sensor,directory)+'/'+self.date_start[:4]+'/...')

                
    def create_pcf ( self,list_pcf ) :

        #pcf_name=DEFAULT_OUTPUT_DIR+'/'+list_pcf['filename']+'.pcf'
        pcf_name=list_pcf['output_id']+'.pcf'
        if self.verbose is True : print 'pcf_name :',pcf_name

        filout = open(pcf_name,'w')
        filout.write('#########################################################################\n')
        filout.write('#\n')
        filout.write('#    PROCESSING COMMANDS FILE - '+list_pcf['sensor']+' - '+list_pcf['prod_id']+'\n')
        filout.write('#\n')
        filout.write('#########################################################################\n')
        filout.write('\n')
        filout.write('#########################################################################\n')
        filout.write('#    '+list_pcf['sensor']+' - RUN PARAMETERS\n')
        filout.write('#########################################################################\n')

        for keys in list_pcf :
            filout.write('\n')
            filout.write('# '+keys)
            filout.write('\n')
            filout.write(keys+'= (')
            for name in list_pcf[keys] :
                if keys=='Input_files':
                    filout.write('\n')
                    filout.write(name+',')
                else :
                    filout.write(name)
                    
            filout.write(')')
            filout.write('\n')
        filout.close()
             
        return pcf_name
        
                      
    def run ( self, prod_id, list_files,time,list_pcf,products) :
        
        """
        run the target application on the given product id for the given timestamp YYYY-MM-DDTHH-MM-SS
        """
        # --- build the common application args
	if time is None :
            name='SEVIRI_AER_OC_'+self.dataset+'_'+self.date_start+'T'+self.start_time[0:2]+'H'+self.start_time[2:4]+'_to_'+self.date_end+'T'+self.end_time[0:2]+'H'+self.end_time[2:4]
            self.date_start+='T'+self.start_time[0:2]+self.start_time[2:4]
            self.date_end+='T'+self.end_time[0:2]+self.end_time[2:4]
            list_pcf['date_start']=self.date_start
            list_pcf['date_end']=self.date_end
            
	if time is not None:
            name='SEVIRI_AER_OC_'+self.dataset+'_'+self.date_start+'_to_'+self.date_end+'_'+self.start_time[0:2]+'H'+self.start_time[2:4]+'_to_'+self.end_time[0:2]+'H'+self.end_time[2:4]
            if self.time_flag == "ORBIT" :
                self.date_start+='T'+self.start_time[0:2]+self.start_time[2:4]
                self.date_end+='T'+self.end_time[0:2]+self.end_time[2:4]
                list_pcf['date_start']=self.date_start
                list_pcf['date_end']=self.date_end
            else :
                list_pcf['date_start']=self.date_start
                list_pcf['date_end']=self.date_end   
        
        if self.output_id is None : self.output_id=name
        list_pcf['output_id']=self.output_id
        
        sensor, directory, pattern = products [ prod_id ]
        list_pcf['sensor']=sensor
        list_pcf['prod_id']=prod_id
	if self.verbose is True : print "Nom pour la sauvegarde du fichier 	:",self.output_id
        cmd_common = "python " + os.path.join ( self.target_app ) +' '
	cmd=cmd_common
	for f in list_files :
	    end=f+' '
	    cmd+=end
        cmd+=' -f '+str(self.output_id)+' -o'+str(self.output_dir)+' -b '+str(self.date_start)+' -c '+str(self.date_end)+' -r '+str(sensor)+' -v '+str(prod_id)+' -d '+str(self.dataset)
        if self.subsetting is not None : cmd+=' -s '+str(self.subsetting[0])+' '+str(self.subsetting[1])+' '+str(self.subsetting[2])+' '+str(self.subsetting[3])
        if self.fixed_value is not None:
            cmd+=' -l '+str(self.fixed_value)
        if self.compress is not True: cmd+=' -g '
        if self.verbose is True :  cmd+=' -e '
        if self.stats is True : cmd+=' -k '
        if self.command_line is True : cmd+=' -y '
             
        #NOMBRE DE CARATRES MAXIMUM SUR UNE LIGNE DE COMMANDE
        nb_car=os.sysconf('SC_ARG_MAX')

        if len(cmd) > 0. and self.command_line is not True :
           print "\n"
           print ' FICHIER DE CONFIG : le nombre de caracteres de la ligne de commande est superieur au nombre supporte par la version linux, nous allons donc utilise un fichier de configuration .pcf '
           print "\n"
           pcf_name=self.create_pcf(list_pcf )
           cmd=cmd_common+pcf_name
           if self.verbose is True :
                print "\n"
                print "Commande du programme Average_processor.py	:",cmd
                print "\n"
           sys.exit(EXIT_SUCCESS)
               
	else:
           if self.verbose is True :
                print "\n"
                print "Commande du programme Average_processor.py	:",cmd
                print "\n"
           STOP
	   os.system(cmd)

    def get_infiles ( self, prod_id,time,products ) :
        """
        construct the list of timestamps of the files that correspond to the product ID and the input time range
        """
	if time is None :
	    if self.time_flag == "ORBIT":
		liste=self.get_list_data(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]
		
	    elif self.time_flag == "DAY" :
		liste=self.get_list_data(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "je suis ici"
		#print "liste	:,",liste,liste[0]

	    elif self.time_flag == "MONTH" :
		liste=self.get_list_data(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]

	    elif self.time_flag == "YEAR" :
		liste=self.get_list_data(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]

	    else :
		raise ValueError ( "Invalid time flag %s"%self.time_flag )
	else :
	    if self.time_flag == "ORBIT":
		liste=self.get_list_data_time(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]
		
	    elif self.time_flag == "DAY" :
		#print "je suis l"
		liste=self.get_list_data_time(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]

	    elif self.time_flag == "MONTH" :
		liste=self.get_list_data_time(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]
		
	    elif self.time_flag == "YEAR" :
		liste=self.get_list_data_time(self.start_time,self.end_time,self.date_start,self.date_end)
		#print "liste	:,",liste,liste[0]

	    else :
		raise ValueError ( "Invalid time flag %s"%self.time_flag )


	list_files = self.get_infiles_day ( prod_id,liste,products )

	return list_files

    def get_infiles_day ( self, prod_id,liste,products ) :
        
        """
        return the list of files of the given product, for the given day
        date_start must have the syntax "YYYY-MM-DD"
        """
        sensor, directory, pattern = products [ prod_id ]
	produit=self.prod_id+'_'
	if self.verbose is True :
            print "Sensor		:",sensor
            print "Directory	:",directory
	
	list_files=[]
	for data in liste :
	   root_dir = os.path.join ( self.input_dir, sensor, directory,data[-1],data[1],produit+data[0]+'.hdf')
           list_files.append(root_dir)
           
        if self.step is not None :
            s=range(1,len(list_files),self.step)
            list_files=delete(list_files,s)

           
        #list_files=get_read(self,list_files)
        if self.verbose is True :
            print "Nombre de fichiers possibles 	:",len(list_files)
            print "\n"
            print "Fichiers	:"
            for files in list_files :
                print files
            print "\n"
        return list_files 

    def get_list_data(self,start_time,end_time,date_start,date_end):
    
	liste=[]
	d1=self.date_start+' '+self.start_time
	ts=datetime(*strptime(d1,"%Y-%m-%d %H%M")[0:5])
	delta=timedelta(minutes=15)
	d2=self.date_end+' '+self.end_time
	te=datetime(*strptime(d2,"%Y-%m-%d %H%M")[0:5])
	te+=delta
	while ts!=te:
	  liste_data=[]
	  t=ts.strftime("%Y%m%d.%H%M")
	  t1=ts.strftime("%Y_%m_%d")
	  t2=ts.year
	  liste_data.append(t)
	  liste_data.append(t1)
	  liste_data.append(str(t2))
	  liste.append(liste_data)
	  ts+=delta

	return liste

    def get_list_data_time(self,start_time,end_time,date_start,date_end):
    
	liste=[]

	h1=self.date_start+' '+self.start_time
	ts=datetime(*strptime(h1,"%Y-%m-%d %H%M")[0:5])
	delta=timedelta(minutes=15)
	h2=self.date_start+' '+self.end_time
	te=datetime(*strptime(h2,"%Y-%m-%d %H%M")[0:5])
	te+=delta
	list_hour=[]
	while ts!=te:
	  t=ts.strftime("%H%M")
	  list_hour.append(t)
	  ts+=delta

        d1=self.date_start
	ts=datetime(*strptime(d1,"%Y-%m-%d")[0:3])
	delta=timedelta(days=1)
	d2=self.date_end
	te=datetime(*strptime(d2,"%Y-%m-%d")[0:3])
	te+=delta
	list_day=[]
	while ts!=te:
	  t=ts.strftime("%Y%m%d.")
	  list_day.append(t)
	  ts+=delta

	for day in list_day :
	    for hour in list_hour :
	      liste_data=[]
	      t=day+hour
	      liste_data.append(t)
	      liste.append(liste_data)
	
	for day in liste :
	    ts=datetime(*strptime(day[0],"%Y%m%d.%H%M")[0:5])
	    t1=ts.strftime("%Y_%m_%d")
	    t2=ts.year
	    day.append(t1)
	    day.append(str(t2))

	return liste

    def get_time_flag ( self, date_start,date_end ) :
    
	if int( date_start[:4])>int( date_end[:4]) :
	    raise ValueError ( "Invalid Date : end date is earlier than the date of the beginning")
	if int( date_start[:4])==int( date_end[:4]) and int( date_start[5:7])>int( date_end[5:7]):
	    raise ValueError ( "Invalid Date : end date is earlier than the date of the beginning")
	if date_start[:7]==date_end[:7] and int( date_start[-2:])>int( date_end[-2:]) :
	    raise ValueError ( "Invalid Date : end date is earlier than the date of the beginning")
	day_start=monthrange(int(date_start[:4]),int(date_start[5:7]))[1]
	day_end=monthrange(int(date_end[:4]),int(date_end[5:7]))[1]
	if int( date_start[8:10])>day_start or int( date_end[8:10])>day_end:
	    raise ValueError ( "Invalid Date : the day  mentioned is wrong ")
	if date_start==date_end :
	     return "ORBIT"
	elif date_start[:7]==date_end[:7]: 
	    return "DAY"
	elif date_start[:4]==date_end[:4] and date_start[5:7]!=date_end[5:7] :
	    return "MONTH"
	else :
	    return "YEAR"
	
    def get_date_start(self,date_start):
    
	if len(date_start)!=4 and len(date_start)!=7 and len(date_start)!=10 and len(date_start)!=15 :
	    raise ValueError ( "Date format invalid")
	if len(date_start)>10 : date_start=date_start[0:10]
	if len(date_start)==4: date_start=date_start+'-01-01'
	if len(date_start)==7: date_start=date_start+'-01'
	    
	return date_start

    def get_date_end(self,date_end,date_start):
    
	if date_end is None : date_end=date_start
	
	if len(date_end)!=4 and len(date_end)!=7 and len(date_end)!=10 and len(date_end)!=15 :
	    raise ValueError ( "Date format invalid")
	if len(date_end)>10 : date_end=date_end[0:10]
	if len(date_end)==4: date_end=date_end+'-12-31'
	if len(date_end)==7: 
	    day=monthrange(int(date_end[:4]),int(date_end[5:7]))[1]
	    date_end=date_end+'-'+str(day)
	return date_end


    def get_struct_time(self,date_start,date_end,time):
    
	Time=[]
        
        if date_end is None : date_end=date_start
        
	if time is not None :
	   Time=time
	elif time is None :
	    if len(date_start)<15 :
                #Time.append('0000')
		Time.append('0400')
	    else :
		Time.append(date_start[11:])
	    if len(date_end)<15 :
		Time.append('2000')
                #Time.append('2345')
	    else :
		Time.append(date_end[11:])

	return Time


    def get_start_time(self,start_time):	
	if len(start_time)!=4 : raise ValueError ( "Start Time format invalid")
	if int(start_time[0:2])>23 : raise ValueError ( "Time format invalid,the number of hours is too high")
	if int(start_time[2:4])>59 : raise ValueError ( "Time format invalid,the number of minutes is too high")
	if int(start_time[2:4])>=0 and int(start_time[2:4])<=7 : start_time=start_time[0:2]+'00'
	elif int(start_time[2:4])>=8 and int(start_time[2:4])<=22 : start_time=start_time[0:2]+'15'
	elif int(start_time[2:4])>=23 and int(start_time[2:4])<=37 : start_time=start_time[0:2]+'30'
	elif int(start_time[2:4])>=38 and int(start_time[2:4])<=52 : start_time=start_time[0:2]+'45'
	elif int(start_time[2:4])>=53 : 
	     h=int(start_time[0:2])+1
	     if len(str(h))==1 : h='0'+str(h)
	     else : h=str(h)
	     start_time=h+'00'
	#if int(start_time[0:2])==24 : start_time='2345'
        if int(start_time[0:2])==24 : start_time='2000'
	return start_time


    def get_end_time(self,end_time):
    
	if len(end_time)!=4 : raise ValueError ( "End Time format invalid")
	if int(end_time[0:2])>23 : raise ValueError ( "Time format invalid,the number of hours is too high")
	if int(end_time[2:4])>59 : raise ValueError ( "Time format invalid,the number of minutes is too high")
	if int(end_time[2:4])>=0 and int(end_time[2:4])<=7 : end_time=end_time[0:2]+'00'
	elif int(end_time[2:4])>=8 and int(end_time[2:4])<=22 : end_time=end_time[0:2]+'15'
	elif int(end_time[2:4])>=23 and int(end_time[2:4])<=37 : end_time=end_time[0:2]+'30'
	elif int(end_time[2:4])>=38 and int(end_time[2:4])<=52 : end_time=end_time[0:2]+'45'
	elif int(end_time[2:4])>=53 : 
	     h=int(end_time[0:2])+1
	     if len(h)==1 : h='0'+str(h)
	     else : h=str(h)
	     end_time=h+'00'
        if int(end_time[0:2])==24 : end_time='2345'
	return end_time
	
    def get_time(self,start_time,end_time,date_start,date_end):
    
	if start_time==end_time and date_start==date_end :
	    raise ValueError ( "Invalid Date : the dates are the same and Time format invalid : there is no period")
	if int(end_time[0:2])<int(start_time[0:2]) and date_start==date_end : 
	    raise ValueError ( "Time format invalid, end_time is wrong")
	if int(end_time[0:2])==int(start_time[0:2]) and int(end_time[2:4])<int(start_time[2:4]) and date_start==date_end :
	    raise ValueError ( "Time format invalid, end_time is wrong")
	return start_time,end_time,date_start,date_end
#
#________________________________________________________________________________________________
#                     
class ProductionParasol :

    def __init__(self, prod_id, date_start,date_end,compress,subsetting,valmax,valmin,verbose,stats,command_line,output_id,step,input_dir = DEFAULT_INPUT_DIR, output_dir = DEFAULT_OUTPUT_DIR):
    
        products=['ATM_WV_D_','ATM_TU_D_','ATM_TAU_D_','ATM_TAUCSM_D_','ATM_TAUCNM_D_','ATM_TAUAFM_D_','ATM_TAUA_D_','ATM_SDWV_D_','ATM_REFFM_D_','ATM_REF_D_','ATM_QCC_D_','ATM_QAVIS_D_','ATM_PRAY_D_','ATM_POXY_D_','ATM_PHASE_D_','ATM_IQAI_D_','ATM_CC_D_','ATM_AVIS_D_','ATM_ASW_D_','ATM_ANGFM_D_','ATM_ANG_D_']

	"""
        Constructor
        @param prod_id a suported PARASOL product ID : one of [ "ATM_ANG_D_", "ATM_ANGFM_D_", "ATM_TAUA_D_" ,"ATM_TAU_D_","ATM_TAU_D_"]
	"""
        self.step=step
        
        self.output_id=output_id
        
        self.verbose=verbose
        
        self.command_line=command_line
        
	self.target_app = os.path.join( os.path.dirname ( __file__ ), "Average_processor.py" )
	
	self.valmax=valmax
        self.valmin=valmin
        
        self.compress=compress
        
        self.stats=stats
        
        self.subsetting=subsetting
        
	self.date_start=self.get_date_start(date_start)

	self.date_end=self.get_date_end(date_end,date_start)

	self.time_flag = self.get_time_flag ( self.date_start,self.date_end )
	
        if self.date_start is not None and self.date_end is not None:
            if self.time_flag == "DAY":  
                list_day=[]
                list_day.append(int(options.date_start[8:10]))
                list_day.append(int(options.date_end[8:10]))
                self.period=list_day
                self.epoch=None
            else :
                list_day=self.get_day(date_start,date_end)
                self.epoch=list_day
                self.period=None
        
	if ( not os.path.exists ( input_dir ) ) :
            raise ValueError ( "Invalid input directory %s"%input_dir )
        self.input_dir  = input_dir
	
	if ( not os.path.exists ( output_dir ) ) :
            raise ValueError ( "Invalid output directory %s"%output_dir )
        self.output_dir    = output_dir

        if prod_id not in products :
            raise ValueError ( "Unsupported Product %s Must be in [%s]"%( prod_id, products ) )
        self.prod_id = prod_id
	
        
        
	if self.verbose is True :
        
            print "\n"
            print "  ######################  FICHIERS POUR MOYENNE IMAGES  #######################"
            print "  #                                                                           #"
            print "  #                                ICARE-CGTD                                 #"
            print "  #                                                                           #"
            print "  #############################################################################"
            print "\n"
            print "  --------------------------------- Parametres --------------------------------"
            print "\n"
            print "\n"
            if self.command_line is not True :
                print "Programme cible		 :",self.target_app
            print "Command_line :",self.command_line
            print "Valmax :",valmax
            print "Valmin :",valmin
            print "Compress :",compress
            print "Stats    :",stats
            print "Subsetting :",subsetting
            print "Dbut de la periode	 :",self.date_start
            print "Fin de la periode	 :",self.date_end
            print "Produit PR_ATM 		 :",self.prod_id
            print "period :",self.period
            print "epoch :",self.epoch
           
            
	v_prod_id = [ self.prod_id ]

	for prod_id in iter ( v_prod_id ) : 
            # build the list of files timestamps matching the time range, and extract the timestamp from the list of files
            list_files,list_day,list_dataset= self.get_infiles ( prod_id,products )
	    
            # run target application on the given product and the given timestamps
	    if len(list_files)!=0 :
              # creation de la liste pour le pcf
              list_pcf={}   
              list_pcf['Input_files']=list_files
              list_pcf['dataset']=list_dataset
              list_pcf['output_directory']=self.output_dir
              list_pcf['compress']=str(self.compress)
              list_pcf['verbose']=str(verbose)
              list_pcf['stats']=str(self.stats)
              list_pcf['command_line']=str(self.command_line)
              
              if self.subsetting is None : list_pcf['subsetting']=str(None)
              else : list_pcf['subsetting']=str(self.subsetting)
              
              list_pcf['date_start']=self.date_start
              list_pcf['date_end']=self.date_end
              
              if self.valmax is None :list_pcf['valmax']=str(None)
              else :list_pcf['valmax']=self.valmax
              
              if self.valmin is None :list_pcf['valmin']=str(None)
              else :list_pcf['valmin']=self.valmin
              
              list_pcf['fixed_value']=str(None)
              
              self.run ( prod_id, list_files, list_day,list_pcf,products)
	    else :
	      #sensor, directory, pattern = products [ prod_id ]
	      raise ValueError ( "No file found in the archive, "+ os.path.join ( self.input_dir,'PARASOL','PR_ATM')+'/'+self.date_start[:4]+'/...')        
    def create_pcf ( self,list_pcf ) :
    
        pcf_name= DEFAULT_OUTPUT_DIR+'/'+list_pcf['output_id']+'.pcf'

        filout = open(pcf_name,'w')
        filout.write('#########################################################################\n')
        filout.write('#\n')
        filout.write('#    PROCESSING COMMANDS FILE - '+list_pcf['sensor']+' - '+list_pcf['prod_id']+'\n')
        filout.write('#\n')
        filout.write('#########################################################################\n')
        filout.write('\n')
        filout.write('#########################################################################\n')
        filout.write('#    '+list_pcf['sensor']+' - RUN PARAMETERS\n')
        filout.write('#########################################################################\n')


        for keys in list_pcf :
            filout.write('\n')
            filout.write('# '+keys)
            filout.write('\n')
            filout.write(keys+'= (')
            if keys=='Input_files' or keys=='dataset':
                l=len(list_pcf['Input_files'])
                i=0
                for name in list_pcf[keys]:
                    i+=1
                    filout.write('\n')
                    if i==l : filout.write(name)
                    else : filout.write(name+',') 
                filout.write(')')
                filout.write('\n')  
            else :
                filout.write(list_pcf[keys][:])
                filout.write(')')
                filout.write('\n')
        filout.close()   
                
    def run ( self, prod_id, list_files,list_day,list_pcf,products) :
        """
        run the target application on the given product id for the given timestamp YYYY-MM-DDTHH-MM-SS
        """
        # --- build the common application args
	name='PR_'+prod_id[:-3]+'_'+self.date_start+'_'+self.date_end
        if self.output_id is None : self.output_id=name
        list_pcf['output_id']=self.output_id
        if self.verbose is True : print "Nom pour la sauvegarde du fichier 	:",self.output_id
        
        
        list_pcf['sensor']='PARASOL'
        list_pcf['prod_id']=prod_id

        cmd_common = "python " + os.path.join ( self.target_app ) +' '
        
        
        if len(list_files)==1 :
            cmd=cmd_common + (list_files[0])+' -f '+str(self.output_id)+' -o'+str(self.output_dir)+' -b '+str(self.date_start)+' -c '+str(self.date_end)+' -r PARASOL'+' -v '+str(prod_id)
            
            if self.subsetting is not None : cmd+=' -s '+str(self.subsetting[0])+' '+str(self.subsetting[1])+' '+str(self.subsetting[2])+' '+str(self.subsetting[3])
            if self.valmax is not None: cmd+=' -m '+str(self.valmax)
            if self.valmin is not None: cmd+=' -n '+str(self.valmin)
            if self.compress is not True: cmd+=' -g '
            if self.verbose is True :  cmd+=' -e '
            if self.stats is True : cmd+=' -k '
            if self.command_line is True : cmd+=' -y '
            
        else :
            cmd=cmd_common
 	    for f in list_files :
		end=f+' '
		cmd+=end
            cmd+=' -f '+str(self.output_id)+ ' -o '+str(self.output_dir)+' -b '+str(self.date_start)+' -c '+str(self.date_end)+' -r PARASOL'+' -v '+str(prod_id)
            if self.subsetting is not None : cmd+=' -s '+str(self.subsetting[0])+' '+str(self.subsetting[1])+' '+str(self.subsetting[2])+' '+str(self.subsetting[3])
            if self.valmax is not None:cmd+=' -m '+str(self.valmax)
            if self.valmin is not None:cmd+=' -n '+str(self.valmin)
            if self.compress is not True:cmd+=' -g '
            if self.verbose is True :  cmd+=' -e '
            if self.stats is True : cmd+=' -k '
            if self.command_line is True : cmd+=' -y '
            
               
        nb_car=os.sysconf('SC_ARG_MAX')

        if len(cmd) > 0. or  self.command_line is not True :
           
           print "\n"
           print ' FICHIER DE CONFIG : le nombre de caracteres de la ligne de commande est superieur au nombre supporte par la version linux, nous allons donc utilise un fichier de configuration .pcf '
           pcf_file=self.create_pcf(list_pcf )
           cmd=cmd_common+DEFAULT_OUTPUT_DIR+'/'+str(self.output_id)+'.pcf'
           if self.verbose is True :
                print "\n"
                print "Commande du programme Average_processor.py	:",cmd
                print "\n"
           

        else :
           if self.verbose is True :
                print "\n"
                print "Commande du programme Average_processor.py	:",cmd
                print "\n"
           STOP
	   os.system(cmd)

    def get_infiles ( self, prod_id,products ) :
        """
        construct the list of timestamps of the files that correspond to the product ID and the input time range
        """
	
        if self.time_flag == "DAY" :
	    liste=[]
	    liste.append(self.date_start[2:4]+self.date_start[5:7])
            liste.append(self.date_start[0:4])
            liste=array(liste,ndmin=2)
	    list_day=[]
	    list_day.append(int(self.date_start[8:10]))
	    list_day.append(int(self.date_end[8:10]))
            list_files,list_dataset = self.get_infiles_day ( prod_id, self.date_start,liste,products )

	elif self.time_flag == "MONTH" :
            liste=self.get_list_data(self.date_start,self.date_end)
	    list_day=self.get_day(self.date_start,self.date_end)
            list_files,list_dataset = self.get_infiles_day(prod_id,self.date_start,liste,products)

        elif self.time_flag == "YEAR" :
            liste=self.get_list_data(self.date_start,self.date_end)
	    list_day=self.get_day(self.date_start,self.date_end)
            list_files,list_dataset = self.get_infiles_day(prod_id,self.date_start,liste,products)
        else :
            raise ValueError ( "Invalid time flag %s"%self.time_flag )
            
        return list_files,list_day,list_dataset
        
    def get_infiles_day ( self, prod_id,date_start,liste,products ) :
        
        
	list_files=[]
	for time in liste :
           #print time
           rep='P3L3TAGP'+time[0]+'15A_v12.07'
           rep_hdf=prod_id+time[0]+'.hdf'
           #rep_hdf=pattern[:-24]+time[0]+'.hdf'
	   year=time[-1]
	   #root_dir = os.path.join ( self.input_dir, sensor, directory,year,rep,pattern)
           root_dir = os.path.join ( self.input_dir,'PARASOL','PR_ATM',year,rep,rep_hdf)
           list_files.append(root_dir)
           
        list_files,list_dataset=get_read(self,list_files)
        
        if self.step is not None :
            s=range(1,len(list_files),self.step)
            list_files=delete(list_files,s)
            list_dataset=delete(list_dataset,s)

            
        if self.verbose is True :
            print "Nombre de fichiers possibles 	:",len(list_files)
            print "\n"
            print "Fichiers	:"
            for files in list_files :
                print files
            print "\n"
            
        return list_files,list_dataset 

    def get_list_data(self,date_end,date_start):
    
        liste=[]
        start=self.date_start[:8]+'01'
        end=self.date_end[:8]+'01'
	day=monthrange(int(end[0:4]),int(end[5:7]))[1]
        
	d1=start
	ts=datetime(*strptime(d1,"%Y-%m-%d")[0:3])
        delta=timedelta(days=day)
	d2=end
	te=datetime(*strptime(d2,"%Y-%m-%d")[0:3])
	te+=delta
        
        while ts!=te : 
          liste_data=[]
          t=ts.strftime("%Y%m")
          t1=int(ts.month)
	  t2=int(ts.year)
          liste_data.append(t[2:])
	  liste_data.append(str(t2))
          liste.append(liste_data)
          day=monthrange(t2,t1)[1]
          delta=timedelta(days=day)
          ts+=delta
        return liste
        

 
    def get_day(self,date_start,date_end):
	star_day=self.date_start[8:10]
	start_month=int(self.date_start[5:7])
	start_year=int(date_start[0:4])
	day=monthrange(start_year,start_month)[1]
	end_day=self.date_end[8:10]
	list_day=[int(star_day),day,1,int(end_day)]
	return list_day

    def get_time_flag ( self, date_start,date_end ) :
        """
        parse the given string timestamp to know idf it represents a day or a month
        @return a string DAY, MONTH or YEAR
        """
	if date_start==date_end :
	    raise ValueError ( "Invalid Date : the dates are the same")
	if int( date_start[:4])>int( date_end[:4]) :
	    raise ValueError ( "Invalid Date : end date is earlier than the date of the beginning")
	if int( date_start[:4])==int( date_end[:4]) and int( date_start[5:7])>int( date_end[5:7]):
	    raise ValueError ( "Invalid Date : end date is earlier than the date of the beginning")
	if date_start[:7]==date_end[:7] and int( date_start[-2:])>int( date_end[-2:]) :
	    raise ValueError ( "Invalid Date : end date is earlier than the date of the beginning")
	day_start=monthrange(int(date_start[:4]),int(date_start[5:7]))[1]
	day_end=monthrange(int(date_end[:4]),int(date_end[5:7]))[1]
	if int( date_start[8:])>day_start or int( date_end[8:])>day_end:
	    raise ValueError ( "Invalid Date : the day  mentioned is wrong ")
	elif date_start[:7]==date_end[:7]: 
	    return "DAY"
	elif date_start[:4]==date_end[:4] and date_start[5:7]!=date_end[5:7] :
	    return "MONTH"
	else :
	    return "YEAR"
	
    def get_date_start(self,date_start):
	
	if len(date_start)!=4 and len(date_start)!=7 and len(date_start)!=10 :
	    raise ValueError ( "Date format invalid")
	if len(date_start)==4: date_start=date_start+'-01-01'
	if len(date_start)==7: date_start=date_start+'-01'
	    
	return date_start

    def get_date_end(self,date_end,date_start):
	if date_end is None : date_end=date_start
	if len(date_end)!=4 and len(date_end)!=7 and len(date_end)!=10 :
	    raise ValueError ( "Date format invalid")
	if len(date_end)==4: date_end=date_end+'-12-31'
	if len(date_end)==7: 
	    day=monthrange(int(date_end[:4]),int(date_end[5:7]))[1]
	    date_end=date_end+'-'+str(day)
	return date_end
#
#________________________________________________________________________________________________
#      
def init_prog_options( parser ):
	"""
	Sets the available program options, and their default values
	"""
	parser.usage = "get_file.py <Product-ID> <dataset> <date of beginning> <end date> [options] \n"
        parser.usage+="\tCan have different formats by sensor:\n"
        parser.usage+="\t To PARASOL :\n"
        parser.usage+="\n  First option :\n "
        parser.usage+="\n  <PR_ATM-ID> : ID of the PR_ATM parameter(s) to process. One of ['ATM_WV_D_','ATM_TU_D_','ATM_TAU_D_','ATM_TAUCSM_D_','ATM_TAUCNM_D_','ATM_TAUAFM_D_','ATM_TAUA_D_','ATM_SDWV_D_','ATM_REFFM_D_','ATM_REF_D_','ATM_QCC_D_','ATM_QAVIS_D_','ATM_PRAY_D_','ATM_POXY_D_','ATM_PHASE_D_','ATM_IQAI_D_','ATM_CC_D_','ATM_AVIS_D_','ATM_ASW_D_','ATM_ANGFM_D_','ATM_ANG_D_'] \n"
	parser.usage+="\n  <date of beginning> and <end date> : timestamp(s) of the input PARASOL files\n" 
        parser.usage+="\t\t- YYYY-MM-DD and  YYYY-MM-DD  : will produce the browses during a given period at daily (Ex : 2007-09-10 2007-09-15)\n"
        parser.usage+="\t\t- YYYY-MM and  YYYY-MM        : will produce the browses for a period up to the monthly or seasonal scale (Ex :2007-09 2007-12)\n"
        parser.usage+="\t\t- YYYY and YYYY               : will produce the browses on an annual basis (Ex : 2006 2008)\n"
        parser.usage+="\n  Second option : "
        parser.usage+="\n  <date> : timestamp(s) of the input PARASOL files\n"
        parser.usage+="\tCan have different formats :\n"
        parser.usage+="\t\t- YYYY-MM                     : will produce the browses for the month specified (Ex: 2008-05)\n"
        parser.usage+="\t\t- YYYY                        : will produce the browses for the year specified (Ex : 2008)\n"
        parser.usage+="\n"
        parser.usage+="\t To SEVIRI :\n"
	parser.usage+="\nArgs:"
	parser.usage+="\n  Two options for the command line : "
	parser.usage+="\n  First option : "
	parser.usage+="\n  <Product-ID> : ID of the SEVIRI product(s) to process : SEVIRI_AER_OC \n"
        parser.usage+="\n  <dataset> : Variable Product SEVIRI, choose between Aerosol_Optical_Depth and Angstrom_Exponent\n"
	parser.usage+="\n  <date of beginning> and <end date> : timestamp(s) of the input SEVIRI files\n"
	parser.usage+="\tCan have different formats :\n"
        parser.usage+="\t\t- YYYY-MM-DDTHHMM and  YYYY-MM-DDTHHMM  : will produce the browses during a given period at daily between hours (Ex : 2007-09-10T0800 2007-09-15T1310)\n"
	parser.usage+="\t\t- YYYY-MM-DD and  YYYY-MM-DD  : will produce the browses during a given period at daily (Ex : 2007-09-10 2007-09-15)\n"
	parser.usage+="\t\t- YYYY-MM and  YYYY-MM        : will produce the browses for a period up to the monthly scale or seasonal scale (Ex :2007-09 2007-12)\n"
	parser.usage+="\t\t- YYYY and YYYY               : will produce the browses on an annual basis (Ex : 2006 2008)\n"
	parser.usage+="\n  Second option : "
        parser.usage+="\n  <dataset> : Variable Product SEVIRI, choose between Aerosol_Optical_Depth and Angstrom_Exponent\n"
	parser.usage+="\n  <Product-ID> : ID of the SEVIRI product(s) to process : SEVIRI_AER_OC \n"
	parser.usage+="\n  <date> : timestamp(s) of the input SEVIRI files\n"
	parser.usage+="\tCan have different formats :\n"
	parser.usage+="\t\t- YYYY-MM-DD                  : will produce the browses for the day specified (Ex: 2008-05-12)\n"
	parser.usage+="\t\t- YYYY-MM                     : will produce the browses for the month specified (Ex: 2008-05)\n"
	parser.usage+="\t\t- YYYY                        : will produce the browses for the year specified (Ex : 2008)\n"
	parser.usage+="\n"
	parser.usage+="It is possible to choose a period of the day as the following example 2009-09-01 2009-09-10 -t 1000 1200\n"
        parser.usage+="For each day, between 10h00 and 12h00 only files are read to calculate the average\n"
	parser.usage+="\n  Subsetting's option : "
        parser.usage+="\n <-s xmin xmax ymin ymax> to select an interest area."
        parser.usage+="\n"
        parser.usage+="\n <-l name of fixed value> shows the fixed values and speeds up the computation time example: space and land values for SEVIRI  ex : -l LAND_VAL,FAILED_VAL. It's different of fill value\n"
        parser.usage+="\n Hdf file compression :"
        
        parser.usage+="\n <-g> Use this option to disable compression"
        parser.usage+="\n  Print program's informations :"
        parser.usage+="\n <-e> Use this option to print program's informations"
        parser.usage+="\n  Additional statistics (standard deviation, min max per pixel:"
        parser.usage+="\n <-k> Use this option to disable obtain additional statistics"
        parser.usage+="\n  Provides the statistical results to png file :"
        parser.usage+="\n <-y> Use this option to execute os.system() and obtain the results to png file"


	parser.add_option("-i","--input_directory",action="store", default=DEFAULT_INPUT_DIR,type="string",dest="input_dir",help="Root of SEVIRI data directory (default:"+DEFAULT_INPUT_DIR+")")
	parser.add_option("-o","--output_directory",action="store", default=DEFAULT_OUTPUT_DIR,type="string",dest="output_dir",help="get_file output directory (default:"+DEFAULT_OUTPUT_DIR+")")
	parser.add_option("-t","--time",
action="store", default=None,type="string",nargs=2,dest="time",help="period chosen for reading data,if it's None all day is read")
        parser.add_option("-s", "--subsetting",type="int",nargs=4,dest="subsetting",default=None,help="This option requires 4 arguments, xmin xmax ymin ymax")
        parser.add_option("-g","--compress",dest="compress",action='store_false',default = True,help="hdf file compression")
        parser.add_option("-k","--stats",dest="stats",action='store_true',default = False,help="Extra statistics calculated : std, min max per pixel")
        parser.add_option("-e","--verbose",dest="verbose",action='store_true',default = False,help="print program's informations")
        parser.add_option("-y","--command_line",dest="command_line",action='store_true',default = False,help="Execute os.system()")
        parser.add_option("-l","--fixed_value",dest="fixed_value",type="string",default = None,help="name of fill value, it is possible that the file has multiple values like SEVIRI (LAND_VAL,SPACE_VAL) ")
        parser.add_option("-m","--valmax",type="string",dest="valmax",default = None)
        parser.add_option("-n","--valmin",type="string",dest="valmin",default = None)
        parser.add_option("-r","--sensor",type="string",dest="sensor",default = None,help="To specify the name of the sensor (PARASOL, MODIS...)")
        parser.add_option("-f", "--output_id",type="string",dest="output_id",default= None,help="give a prefix of output file for HDF, PCf or PNG format")
        parser.add_option("-p", "--step",type="int",nargs=1,dest="step",default=None,help="allows to choose the step of list of files")

	if len ( sys.argv ) < 2 :
	    parser.print_help ( )
	    raise ValueError ( "\nMissing command-line options" )
	if len ( sys.argv ) > 26 :
	    parser.print_help ( )
	    raise ValueError ( "\n Too many arguments and options in command-line" )

#
#________________________________________________________________________________________________
#            
def get_read(self,list_files):
    
    list_files=sort(list_files)
    list_dataset=[]
    file_list=[]
    for file in  list_files:
       hdf_name=file
       hdf_file = SD(hdf_name)
       datasets=hdf_file.datasets()
       keys=datasets.keys()
       if self.period is not None :
            keys=sort(keys)[int(self.period[0])-1:int(self.period[1])]
            for K in keys :
                list_dataset.append(K)
                file_list.append(file)
       if self.epoch is not None :
            keys=sort(keys)
            if file==list_files[0]:
                keys=keys[int(self.epoch[0])-1:int(self.epoch[1])]
                for K in keys : 
                    list_dataset.append(K)
                    file_list.append(file)
            elif file==list_files[-1]:
                keys=keys[int(self.epoch[2])-1:int(self.epoch[-1])]
                for K in keys : 
                    list_dataset.append(K)
                    file_list.append(file)
            else :
                keys=keys
                for K in keys : 
                    list_dataset.append(K)
                    file_list.append(file)
    list_files= file_list           
    return list_files,list_dataset
#
#________________________________________________________________________________________________
#
def main():
        start_time=time()
	#print "\n   ---------- DEBUT DU PROGRAMME -------------\n"
	# parse the command line arguments
	parser = OptionParser()
	init_prog_options ( parser )
	( options, args ) = parser.parse_args ( )

        if options.sensor is None :
            msg="Error : the sensor's name must be indicated. Example : -r SEVIRI, -r PARASOL, -r MODIS, ..."
	    raise ValueError(msg)
        
        if options.sensor=='SEVIRI' :            
            if len(args)==3:
                prog=ProductionSeviri(args[0],args[1],args[2],date_end=None,time=options.time,input_dir=options.input_dir, output_dir= options.output_dir,subsetting=options.subsetting,compress=options.compress,verbose=options.verbose,stats=options.stats,command_line=options.command_line,fixed_value=options.fixed_value,output_id=options.output_id,step=options.step)
            elif len(args)==4 :
                prog=ProductionSeviri(args[0], args[1],args[2],args[3],time=options.time,input_dir=options.input_dir, output_dir= options.output_dir,subsetting=options.subsetting,compress=options.compress,verbose=options.verbose,stats=options.stats,command_line=options.command_line,fixed_value=options.fixed_value,output_id=options.output_id,step=options.step)
            else :
                raise ValueError ( "Too many arguments")
        elif options.sensor=='PARASOL' :  
            if len ( args ) == 2 :
                prog=ProductionParasol(args[0], args[1],date_end=None,compress=options.compress,subsetting=options.subsetting,valmax=options.valmax,valmin=options.valmin,verbose=options.verbose,stats=options.stats,command_line=options.command_line, input_dir =options.input_dir, output_dir= options.output_dir,output_id=options.output_id,step=options.step)
            elif len ( args ) == 3 :
                prog=ProductionParasol(args[0], args[1], args[2],compress=options.compress,subsetting=options.subsetting,valmax=options.valmax,valmin=options.valmin,verbose=options.verbose,stats=options.stats,command_line=options.command_line, input_dir =options.input_dir, output_dir= options.output_dir,output_id=options.output_id,step=options.step)
            else :
                raise ValueError ( "Too many arguments")
        else :
            raise ValueError ( "the sensor must be SEVIRI or PARASOL")
	#all works good
	print "   ---------- end of PROGRAMME with status "+str(EXIT_SUCCESS)+" (elapsed time : "+strftime("%H:%M:%S",gmtime(time()-start_time))+") -------------"
	sys.exit(EXIT_SUCCESS)

if __name__ == "__main__":
    main()